import colorful as cf

logo = '''{c.bold}{c.pink}\
              __              __             _     
  ____  _____/ /_____        / /__________ _(_)___ 
 / __ \/ ___/ __/ __ \______/ __/ ___/ __ `/ / __ \\
/ /_/ / /__/ /_/ /_/ /_____/ /_/ /  / /_/ / / / / /
\____/\___/\__/\____/      \__/_/   \__,_/_/_/ /_/ {c.reset}'''.format(c=cf)

welcome_message = f'''\
                    Welcome to
{logo}
An individual programming trainer for everyone.'''


def multisolve(solved, goal):
    diff = solved - goal
    if diff < 0:
        return ''
    phrases = [
        '',
        'Dominating',
        'Rampage',
        'Mega Solve',
        'Unstoppable',
        'Wicked Sick',
        'Monster Solve',
        'Godlike',
        'Beyond Godlike',
    ]
    phrase = ' {c.yellow}(Goal completed){c.bold}{c.red} '.format(c=cf)
    phrase += phrases[min(diff, len(phrases) - 1)].upper() + str(cf.reset)
    return phrase



import requests
from bs4 import BeautifulSoup
from random import randint

from octo_train.source_interface import Problem, ProblemSource

BASE_URL = 'https://codeforces.com'


class CFProblem(Problem):
    def json(self):
        d = super().json()
        d['problem_type'] = 'cf'
        return d


class CodeForces(ProblemSource):
    def get_problem(self):
        give_not_solved = randint(1, 100) <= 33
        if give_not_solved:
            problem = self.db.get_not_solved_problem('cf')
            if problem is not None:
                return CFProblem(problem['title'], problem['difficulty'], problem['link'])

        difficulty = int(self.db.user['cf_level'])
        solved = self.db.get_all('cf')
        start_page = self.db.user['cf_page']
        for i in range(start_page, 61):
            url = f'{BASE_URL}/problemset/page/{i}?order=BY_RATING_ASC'
            page = requests.get(url)
            soup = BeautifulSoup(page.text, 'html.parser')

            problems = soup.find('table', 'problems')('tr')
            for problem in problems:
                difficulty_cell = problem.select('td > span.ProblemRating')
                if not difficulty_cell:
                    continue
                problem_difficulty = int(difficulty_cell[0].text)
                if problem_difficulty < difficulty:
                    continue
                title_cell = problem.select('td > div > a')
                if not title_cell:
                    continue
                title_cell = title_cell[0]
                link = BASE_URL + title_cell['href']
                if link in solved:
                    continue
                title = title_cell.text.replace('\n', '').rstrip(' ')

                if i > start_page:
                    self.db.update_user('cf_page', i)

                if problem_difficulty > difficulty:
                    self.db.update_user('cf_level', problem_difficulty)

                return CFProblem(title, problem_difficulty, link)

    def solved_today(self):
        return int(self.db.solved_today('cf'))

    def goal(self):
        return int(self.db.user['cf_daily'])



import os

from octo_train.db import Database
from octo_train.codeforces import CodeForces
from octo_train.euler import ProjectEuler
from octo_train.colored import welcome_message, logo, multisolve


menu = [
    'CodeForces',
    'ProjectEuler',
    'Stats',
    'Settings',
    'Quit'
]


def show_menu():
    for i, item in enumerate(menu):
        print(i + 1, ') ', item, sep='')
    while True:
        inp = input()
        if len(inp) == 1 and inp.isdigit():
            return menu[int(inp) - 1]


def clear():
    if os.name == 'posix':
        os.system('clear')
    else:
        os.system('cls')


def input_to_bool(inp):
    inp = inp.lower()
    t = ['yes', 'y', 'н', 'нуы', 'д', 'да']
    return inp in t


def main():
    clear()
    db = Database()
    if not db.loaded:
        print(welcome_message)
        name = input('Enter your name: ')
        cf_level = input('Enter your CodeForces level (help for additional info): ')
        cf_daily = input('Enter number of CodeForces problems you will solve per day: ')
        pe_level = input('Enter your ProjectEuler level (help for additional info): ')
        pe_daily = input('Enter number of ProjectEuler problems you will solve per day: ')
        db.new_user(name=name, cf_level=cf_level, pe_level=pe_level, cf_daily=cf_daily, pe_daily=pe_daily, cf_page=2)
    cf = CodeForces(db)
    pe = ProjectEuler(db)
    while True:
        clear()
        print(logo)
        print(f'Welcome back, {db.user["name"]}!')
        print('---------')
        print('Your today goals:')
        cf_solved, cf_goal = cf.solved_today(), cf.goal()
        if cf.goal() != 0:
            print(f'CodeForces - {cf_solved}/{cf_goal}{multisolve(cf_solved, cf_goal)}')
        pe_solved, pe_goal = pe.solved_today(), pe.goal()
        if pe.goal() != 0:
            print(f'ProjectEuler - {pe_solved}/{pe_goal}{multisolve(pe_solved, pe_goal)}')
        print('---------')

        choose = show_menu()
        if choose == 'CodeForces':
            clear()
            print(logo)
            problem = cf.get_problem()
            print(problem.title, f'({problem.difficulty})')
            print(problem.link)
            solved = input_to_bool(input('Solved? (y/n): '))
            if solved:
                problem.solved = True
            cf.add_problem_to_db(problem)
        elif choose == 'ProjectEuler':
            clear()
            print(logo)
            print('NOT DONE YET (Press ENTER to continue...)')
            input()
        elif choose == 'Stats':
            clear()
            print(logo)
            print('---------')
            print('All time stats:')
            print('---------')
            print(f'CodeForces: {len(db.get_solved("cf"))} (solved), {len(db.get_not_solved("cf"))} (not solved)')
            print(f'ProjectEuler: {len(db.get_solved("pe"))} (solved), {len(db.get_not_solved("pe"))} (not solved)')
            print('---------')
            input('Press ENTER to go back...')
        elif choose == 'Settings':
            while True:
                clear()
                print(logo)
                print('\n'.join([
                    f'1) Change name ({db.user["name"]})',
                    f'2) Change CodeForces level ({db.user["cf_level"]})',
                    f'3) Change CodeForces daily goal ({db.user["cf_daily"]})',
                    f'4) Change ProjectEuler level ({db.user["pe_level"]})',
                    f'5) Change ProjectEuler daily goal ({db.user["pe_daily"]})',
                    f'6) Back']))
                s_choose = input()
                if len(s_choose) != 1 or not s_choose.isdigit():
                    continue
                s_choose = int(s_choose)
                if s_choose == 1:
                    db.update_user('name', input('New name: '))
                elif s_choose == 2:
                    db.update_user('cf_level', input('New CF level: '))
                elif s_choose == 3:
                    db.update_user('cf_daily', input('New CF daily goal: '))
                elif s_choose == 4:
                    db.update_user('pe_level', input('New PE level: '))
                elif s_choose == 5:
                    db.update_user('pe_daily', input('New PE daily goal: '))
                elif s_choose == 6:
                    break
        elif choose == 'Quit':
            clear()
            break


if __name__ == '__main__':
    main()



import requests
from bs4 import BeautifulSoup
from random import randint

from octo_train.source_interface import Problem, ProblemSource

BASE_URL = 'https://projecteuler.net'


class PEProblem(Problem):
    def json(self):
        d = super().json()
        d['problem_type'] = 'pe'
        return d


class ProjectEuler(ProblemSource):
    def get_problem(self):
        give_not_solved = randint(1, 100) <= 33
        if give_not_solved:
            problem = self.db.get_not_solved_problem('pe')
            if problem is not None:
                return PEProblem(problem['title'], problem['difficulty'], problem['link'])

        difficulty = self.db.user['pe_level']
        solved = self.db.get_all('pe')

    def solved_today(self):
        return int(self.db.solved_today('pe'))

    def goal(self):
        return int(self.db.user['pe_daily'])



from abc import ABC, abstractmethod
from octo_train.db import Database


class Problem(ABC):
    def __init__(self, title, difficulty, link):
        self.title = title
        self.difficulty = difficulty
        self.link = link
        self.solved = False

    @abstractmethod
    def json(self):
        return {
            'title': self.title,
            'difficulty': self.difficulty,
            'link': self.link,
            'solved': self.solved
        }


class ProblemSource(ABC):
    def __init__(self, db: Database):
        self.db = db

    def add_problem_to_db(self, task):
        self.db.insert(task.json())

    @abstractmethod
    def get_problem(self):
        pass

    @abstractmethod
    def solved_today(self):
        pass

    @abstractmethod
    def goal(self):
        pass



import os
from datetime import datetime
from random import randint
from tinydb import TinyDB, where

DATE_FORMAT = '%d.%m.%y'

PATH_TO_DB = os.path.expanduser('~/octo_train_data.json')


class Database:
    def __init__(self):
        self.db = TinyDB(PATH_TO_DB)
        self.user = None
        self.loaded = False
        if self.db.contains(where('name')):
            self.loaded = True
            self.user = self._get_user()

    def new_user(self, **kwargs):
        self.db.insert(kwargs)
        self.user = self._get_user()

    def get_stats(self):
        pass

    def solved_today(self, problem_type):
        today = self._get_today_date()
        st = self.db.search((where('date') == today)
                            & (where('problem_type') == problem_type)
                            & (where('solved') == True))
        return len(st)

    def get_solved(self, problem_type):
        l = self.db.search((where('problem_type') == problem_type) & (where('solved') == True))
        result = set()
        for item in l:
            result.add(item['link'])
        return result

    def get_not_solved(self, problem_type):
        l = self.db.search((where('problem_type') == problem_type) & (where('solved') == False))
        result = set()
        for item in l:
            result.add(item['link'])
        return result

    def get_all(self, problem_type):
        l = self.db.search(where('problem_type') == problem_type)
        result = set()
        for item in l:
            result.add(item['link'])
        return result

    def get_not_solved_problem(self, problem_type):
        l = self.db.search((where('problem_type') == problem_type) & (where('solved') == False))
        if not l:
            return None
        problem_number = randint(0, len(l) - 1)
        problem = l[problem_number]
        self.db.remove(where('link') == problem['link'])
        return problem

    def insert(self, data):
        data['date'] = self._get_today_date()
        self.db.insert(data)

    def update_user(self, key, value):
        self.user[key] = value
        self.db.write_back([self.user])

    def _get_user(self):
        try:
            user = self.db.search(where('name'))[0]
        except IndexError:
            raise
        else:
            return user

    def _get_today_date(self):
        return datetime.now().strftime(DATE_FORMAT)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='octo_train',
      description='Console application to make you a great programmer.',
      version='0.1.2',
      packages=find_packages(),
      entry_points={
          'console_scripts': [
              'octo-train = octo_train.main:main'
          ]},
      install_requires=['tinydb>=3.15.2', 'colorful', 'requests', 'beautifulsoup4'],
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='dankondr',
      author_email="dankondr@icloud.com",
      url="https://github.com/dankondr/octo-train",
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
      ],
      python_requires='>=3.7'
      )
